@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 the Pacemaker project contributors
+ * Copyright 2010-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -24,7 +24,7 @@
 #include <crm/crm.h>  /* indirectly: CRM_EX_* */
 #include <crm/cib/internal.h>  /* cib_channel_ro */
 #include <crm/msg_xml.h>
-#include <crm/common/ipcs.h>
+#include <crm/common/ipcs_internal.h>
 #include <crm/common/mainloop.h>
 #include <crm/cluster/internal.h>
 #include <crm/cluster.h>
@@ -51,7 +51,7 @@ static crm_trigger_t *shutdown_trigger = NULL;
 static const char *pid_file = PCMK_RUN_DIR "/pacemaker.pid";
 
 typedef struct pcmk_child_s {
-    int pid;
+    pid_t pid;
     long flag;
     int start_seq;
     int respawn_count;
@@ -80,7 +80,7 @@ static pcmk_child_t pcmk_children[] = {
     {
         0, crm_proc_based,      1, 0, TRUE,  "pacemaker-based",
         CRM_DAEMON_USER, CRM_DAEMON_DIR "/pacemaker-based",
-        CIB_CHANNEL_RO
+        PCMK__SERVER_BASED_RO
     },
     {
         0, crm_proc_controld,   6, 0, TRUE, "pacemaker-controld",
@@ -105,11 +105,11 @@ static pcmk_child_t pcmk_children[] = {
 };
 
 static gboolean check_active_before_startup_processes(gpointer user_data);
-static int pcmk_child_active(pcmk_child_t *child);
+static int child_liveness(pcmk_child_t *child);
 static gboolean start_child(pcmk_child_t * child);
 static gboolean update_node_processes(uint32_t id, const char *uname,
                                       uint32_t procs);
-void update_process_clients(crm_client_t *client);
+void update_process_clients(pcmk__client_t *client);
 
 static uint32_t
 get_process_list(void)
@@ -160,7 +160,7 @@ pcmk_process_exit(pcmk_child_t * child)
         crm_err("Rebooting system because of %s", child->name);
         pcmk_panic(__FUNCTION__);
 
-    } else if (pcmk_child_active(child) == 1) {
+    } else if (child_liveness(child) == pcmk_rc_ok) {
         crm_warn("One-off suppressing strict respawning of a child process %s,"
                  " appears alright per %s IPC end-point",
                  child->name, child->endpoint);
@@ -227,9 +227,7 @@ pcmk_child_exit(mainloop_child_t * p, pid_t pid, int core, int signo, int exitco
                 break;
 
             case CRM_EX_PANIC:
-                do_crm_log_always(LOG_EMERG,
-                                  "%s[%d] instructed the machine to reset",
-                                  name, pid);
+                crm_emerg("%s[%d] instructed the machine to reset", name, pid);
                 child->respawn = FALSE;
                 fatal_error = TRUE;
                 pcmk_panic(__FUNCTION__);
@@ -270,12 +268,12 @@ stop_child(pcmk_child_t * child, int signal)
 
     errno = 0;
     if (kill(child->pid, signal) == 0) {
-        crm_notice("Stopping %s "CRM_XS" sent signal %d to process %d",
-                   child->name, signal, child->pid);
+        crm_notice("Stopping %s "CRM_XS" sent signal %d to process %lld",
+                   child->name, signal, (long long) child->pid);
 
     } else {
-        crm_perror(LOG_ERR, "Could not stop %s (process %d) with signal %d",
-                   child->name, child->pid, signal);
+        crm_err("Could not stop %s (process %lld) with signal %d: %s",
+                child->name, (long long) child->pid, signal, strerror(errno));
     }
 
     return TRUE;
@@ -287,7 +285,7 @@ static char *opts_vgrind[] = { NULL, NULL, NULL, NULL, NULL };
 /* TODO once libqb is taught to juggle with IPC end-points carried over as
         bare file descriptor (https://github.com/ClusterLabs/libqb/issues/325)
         it shall hand over these descriptors here if/once they are successfully
-        pre-opened in (presumably) pcmk_child_active, to avoid any remaining
+        pre-opened in (presumably) child_liveness(), to avoid any remaining
         room for races */
 static gboolean
 start_child(pcmk_child_t * child)
@@ -296,7 +294,6 @@ start_child(pcmk_child_t * child)
     gid_t gid = 0;
     gboolean use_valgrind = FALSE;
     gboolean use_callgrind = FALSE;
-    const char *devnull = "/dev/null";
     const char *env_valgrind = getenv("PCMK_valgrind_enabled");
     const char *env_callgrind = getenv("PCMK_callgrind_enabled");
 
@@ -343,7 +340,8 @@ start_child(pcmk_child_t * child)
         /* parent */
         mainloop_child_add(child->pid, 0, child->name, child, pcmk_child_exit);
 
-        crm_info("Forked child %d for process %s%s", child->pid, child->name,
+        crm_info("Forked child %lld for process %s%s",
+                 (long long) child->pid, child->name,
                  use_valgrind ? " (valgrind enabled: " VALGRIND_BIN ")" : "");
         update_node_processes(local_nodeid, NULL, get_process_list());
         return TRUE;
@@ -398,9 +396,9 @@ start_child(pcmk_child_t * child)
 
         pcmk__close_fds_in_child(true);
 
-        (void)open(devnull, O_RDONLY);  /* Stdin:  fd 0 */
-        (void)open(devnull, O_WRONLY);  /* Stdout: fd 1 */
-        (void)open(devnull, O_WRONLY);  /* Stderr: fd 2 */
+        pcmk__open_devnull(O_RDONLY);   // stdin (fd 0)
+        pcmk__open_devnull(O_WRONLY);   // stdout (fd 1)
+        pcmk__open_devnull(O_WRONLY);   // stderr (fd 2)
 
         if (use_valgrind) {
             (void)execvp(VALGRIND_BIN, opts_vgrind);
@@ -422,7 +420,7 @@ escalate_shutdown(gpointer data)
     if (child->pid == PCMK__SPECIAL_PID) {
         pcmk_process_exit(child);
 
-    } else if (child->pid) {
+    } else if (child->pid != 0) {
         /* Use SIGSEGV instead of SIGKILL to create a core so we can see what it was up to */
         crm_err("Child %s not terminating in a timely manner, forcing", child->name);
         stop_child(child, SIGSEGV);
@@ -456,7 +454,7 @@ pcmk_shutdown_worker(gpointer user_data)
                 continue;
             }
 
-            if (child->pid) {
+            if (child->pid != 0) {
                 time_t now = time(NULL);
 
                 if (child->respawn) {
@@ -483,8 +481,9 @@ pcmk_shutdown_worker(gpointer user_data)
                 } else if (now >= next_log) {
                     next_log = now + 30;
                     crm_notice("Still waiting for %s to terminate "
-                               CRM_XS " pid=%d seq=%d",
-                               child->name, child->pid, child->start_seq);
+                               CRM_XS " pid=%lld seq=%d",
+                               child->name, (long long) child->pid,
+                               child->start_seq);
                 }
                 return TRUE;
             }
@@ -499,7 +498,7 @@ pcmk_shutdown_worker(gpointer user_data)
     crm_notice("Shutdown complete");
 
     {
-        const char *delay = daemon_option("shutdown_delay");
+        const char *delay = pcmk__env_option("shutdown_delay");
         if(delay) {
             sync();
             sleep(crm_get_msec(delay) / 1000);
@@ -541,18 +540,12 @@ static int32_t
 pcmk_ipc_accept(qb_ipcs_connection_t * c, uid_t uid, gid_t gid)
 {
     crm_trace("Connection %p", c);
-    if (crm_client_new(c, uid, gid) == NULL) {
+    if (pcmk__new_client(c, uid, gid) == NULL) {
         return -EIO;
     }
     return 0;
 }
 
-static void
-pcmk_ipc_created(qb_ipcs_connection_t * c)
-{
-    crm_trace("Connection %p", c);
-}
-
 /* Exit code means? */
 static int32_t
 pcmk_ipc_dispatch(qb_ipcs_connection_t * qbc, void *data, size_t size)
@@ -560,10 +553,10 @@ pcmk_ipc_dispatch(qb_ipcs_connection_t * qbc, void *data, size_t size)
     uint32_t id = 0;
     uint32_t flags = 0;
     const char *task = NULL;
-    crm_client_t *c = crm_client_get(qbc);
-    xmlNode *msg = crm_ipcs_recv(c, data, size, &id, &flags);
+    pcmk__client_t *c = pcmk__find_client(qbc);
+    xmlNode *msg = pcmk__client_data2xml(c, data, &id, &flags);
 
-    crm_ipcs_send_ack(c, id, flags, "ack", __FUNCTION__, __LINE__);
+    pcmk__ipc_send_ack(c, id, flags, "ack");
     if (msg == NULL) {
         return 0;
     }
@@ -602,13 +595,13 @@ pcmk_ipc_dispatch(qb_ipcs_connection_t * qbc, void *data, size_t size)
 static int32_t
 pcmk_ipc_closed(qb_ipcs_connection_t * c)
 {
-    crm_client_t *client = crm_client_get(c);
+    pcmk__client_t *client = pcmk__find_client(c);
 
     if (client == NULL) {
         return 0;
     }
     crm_trace("Connection %p", c);
-    crm_client_destroy(client);
+    pcmk__free_client(client);
     return 0;
 }
 
@@ -621,12 +614,19 @@ pcmk_ipc_destroy(qb_ipcs_connection_t * c)
 
 struct qb_ipcs_service_handlers mcp_ipc_callbacks = {
     .connection_accept = pcmk_ipc_accept,
-    .connection_created = pcmk_ipc_created,
+    .connection_created = NULL,
     .msg_process = pcmk_ipc_dispatch,
     .connection_closed = pcmk_ipc_closed,
     .connection_destroyed = pcmk_ipc_destroy
 };
 
+static void
+send_xml_to_client(gpointer key, gpointer value, gpointer user_data)
+{
+    pcmk__ipc_send_xml((pcmk__client_t *) value, 0, (xmlNode *) user_data,
+                       crm_ipc_server_event);
+}
+
 /*!
  * \internal
  * \brief Send an XML message with process list of all known peers to client(s)
@@ -634,7 +634,7 @@ struct qb_ipcs_service_handlers mcp_ipc_callbacks = {
  * \param[in] client  Send message to this client, or all clients if NULL
  */
 void
-update_process_clients(crm_client_t *client)
+update_process_clients(pcmk__client_t *client)
 {
     GHashTableIter iter;
     crm_node_t *node = NULL;
@@ -656,16 +656,13 @@ update_process_clients(crm_client_t *client)
 
     if(client) {
         crm_trace("Sending process list to client %s", client->id);
-        crm_ipcs_send(client, 0, update, crm_ipc_server_event);
+        send_xml_to_client(NULL, client, update);
 
     } else {
-        crm_trace("Sending process list to %d clients", crm_hash_table_size(client_connections));
-        g_hash_table_iter_init(&iter, client_connections);
-        while (g_hash_table_iter_next(&iter, NULL, (gpointer *) & client)) {
-            crm_ipcs_send(client, 0, update, crm_ipc_server_event);
-        }
+        crm_trace("Sending process list to %d clients",
+                  pcmk__ipc_client_count());
+        pcmk__foreach_ipc_client(send_xml_to_client, update);
     }
-
     free_xml(update);
 }
 
@@ -730,23 +727,48 @@ update_node_processes(uint32_t id, const char *uname, uint32_t procs)
 }
 
 
-/* *INDENT-OFF* */
-static struct crm_option long_options[] = {
-    /* Top-level Options */
-    {"help",           0, 0, '?', "\tThis text"},
-    {"version",        0, 0, '$', "\tVersion information"  },
-    {"verbose",        0, 0, 'V', "\tIncrease debug output"},
-    {"shutdown",       0, 0, 'S', "\tInstruct Pacemaker to shutdown on this machine"},
-    {"features",       0, 0, 'F', "\tDisplay the full version and list of features Pacemaker was built with"},
-
-    {"-spacer-",       1, 0, '-', "\nAdditional Options:"},
-    {"foreground",     0, 0, 'f', "\t(Ignored) Pacemaker always runs in the foreground"},
-    {"pid-file",       1, 0, 'p', "\t(Ignored) Daemon pid file location"},
-    {"standby",        0, 0, 's', "\tStart node in standby state"},
-
-    {NULL, 0, 0, 0}
+static pcmk__cli_option_t long_options[] = {
+    // long option, argument type, storage, short option, description, flags
+    {
+        "help", no_argument, NULL, '?',
+        "\tThis text", pcmk__option_default
+    },
+    {
+        "version", no_argument, NULL, '$',
+        "\tVersion information", pcmk__option_default
+    },
+    {
+        "verbose", no_argument, NULL, 'V',
+        "\tIncrease debug output", pcmk__option_default
+    },
+    {
+        "shutdown", no_argument, NULL, 'S',
+        "\tInstruct Pacemaker to shutdown on this machine", pcmk__option_default
+    },
+    {
+        "features", no_argument, NULL, 'F',
+        "\tDisplay full version and list of features Pacemaker was built with",
+        pcmk__option_default
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "\nAdditional Options:", pcmk__option_default
+    },
+    {
+        "foreground", no_argument, NULL, 'f',
+        "\t(Ignored) Pacemaker always runs in the foreground",
+        pcmk__option_default
+    },
+    {
+        "pid-file", required_argument, NULL, 'p',
+        "\t(Ignored) Daemon pid file location", pcmk__option_default
+    },
+    {
+        "standby", no_argument, NULL, 's',
+        "\tStart node in standby state", pcmk__option_default
+    },
+    { 0, 0, 0, 0 }
 };
-/* *INDENT-ON* */
 
 static void
 mcp_chown(const char *path, uid_t uid, gid_t gid)
@@ -765,94 +787,126 @@ mcp_chown(const char *path, uid_t uid, gid_t gid)
  *
  * \param[inout] child  Child tracked data
  *
- * \return 0 if no trace of child's liveness detected (while it is detectable
- *         to begin with, at least according to one of the two properties),
- *         1 if everything is fine, 2 if it's up per PID, but not per IPC
- *         end-point (still starting?), -1 on error, and -2 when the child
- *         (its IPC) blocked with an unauthorized process (log message
- *         emitted in both latter cases)
+ * \return Standard Pacemaker return code
  *
+ * \note Return codes of particular interest include pcmk_rc_ipc_unresponsive
+ *       indicating that no trace of IPC liveness was detected,
+ *       pcmk_rc_ipc_unauthorized indicating that the IPC endpoint is blocked by
+ *       an unauthorized process, and pcmk_rc_ipc_pid_only indicating that
+ *       the child is up by PID but not IPC end-point (possibly starting).
  * \note This function doesn't modify any of \p child members but \c pid,
  *       and is not actively toying with processes as such but invoking
  *       \c stop_child in one particular case (there's for some reason
  *       a different authentic holder of the IPC end-point).
  */
 static int
-pcmk_child_active(pcmk_child_t *child) {
-    static uid_t cl_uid = 0;
-    static gid_t cl_gid = 0;
+child_liveness(pcmk_child_t *child)
+{
+    uid_t cl_uid = 0;
+    gid_t cl_gid = 0;
     const uid_t root_uid = 0;
     const gid_t root_gid = 0;
     const uid_t *ref_uid;
     const gid_t *ref_gid;
-    int ret = 0;
+    int rc = pcmk_rc_ipc_unresponsive;
     pid_t ipc_pid = 0;
 
     if (child->endpoint == NULL
             && (child->pid <= 0 || child->pid == PCMK__SPECIAL_PID)) {
         crm_err("Cannot track child %s for missing both API end-point and PID",
                 child->name);
-        ret = -1;  /* misuse of the function when child is not trackable */
+        rc = EINVAL; // Misuse of function when child is not trackable
 
     } else if (child->endpoint != NULL) {
+        int legacy_rc = pcmk_ok;
 
-        ref_uid = (child->uid != NULL) ? &cl_uid : &root_uid;
-        ref_gid = (child->uid != NULL) ? &cl_gid : &root_gid;
+        if (child->uid == NULL) {
+            ref_uid = &root_uid;
+            ref_gid = &root_gid;
+        } else {
+            ref_uid = &cl_uid;
+            ref_gid = &cl_gid;
+            legacy_rc = pcmk_daemon_user(&cl_uid, &cl_gid);
+        }
 
-        if (child->uid != NULL && !cl_uid && !cl_gid
-                && crm_user_lookup(CRM_DAEMON_USER, &cl_uid, &cl_gid) < 0) {
-            crm_err("Could not find user and group IDs for user %s",
-                    CRM_DAEMON_USER);
-            ret = -1;
-        } else if ((ret = pcmk__ipc_is_authentic_process_active(child->endpoint,
-                                                                *ref_uid, *ref_gid,
-                                                                &ipc_pid)) < 0) {
-            /* game over */
-        } else if (child->pid <= 0) {
-            /* hit new child to be initialized, or reset to zero
-               and investigate further for ret == 0 */
-            child->pid = ipc_pid;
-        } else if (ipc_pid && child->pid != ipc_pid) {
-            /* ultimately strange for ret == 1; either way, investigate */
-            ret = 0;
+        if (legacy_rc < 0) {
+            rc = pcmk_legacy2rc(legacy_rc);
+            crm_err("Could not find user and group IDs for user %s: %s "
+                    CRM_XS " rc=%d", CRM_DAEMON_USER, pcmk_rc_str(rc), rc);
+        } else {
+            rc = pcmk__ipc_is_authentic_process_active(child->endpoint,
+                                                       *ref_uid, *ref_gid,
+                                                       &ipc_pid);
+            if ((rc == pcmk_rc_ok) || (rc == pcmk_rc_ipc_unresponsive)) {
+                if (child->pid <= 0) {
+                    /* If rc is pcmk_rc_ok, ipc_pid is nonzero and this
+                     * initializes a new child. If rc is
+                     * pcmk_rc_ipc_unresponsive, ipc_pid is zero, and we will
+                     * investigate further.
+                     */
+                    child->pid = ipc_pid;
+                } else if ((ipc_pid != 0) && (child->pid != ipc_pid)) {
+                    /* An unexpected (but authorized) process is responding to
+                     * IPC. Investigate further.
+                     */
+                    rc = pcmk_rc_ipc_unresponsive;
+                }
+            }
         }
     }
 
-    if (!ret) {
-        /* when no IPC based liveness detected (incl. if ever a child without
-           IPC is tracked), or detected for a different _authentic_ process;
-           safe on FreeBSD since the only change possible from a proper child's
-           PID into "special" PID of 1 behind more loosely related process */
-        ret = crm_pid_active(child->pid, child->name);
-        if (ipc_pid && (ret != 1
+    if (rc == pcmk_rc_ipc_unresponsive) {
+        /* If we get here, a child without IPC is being tracked, no IPC liveness
+         * has been detected, or IPC liveness has been detected with an
+         * unexpected (but authorized) process. This is safe on FreeBSD since
+         * the only change possible from a proper child's PID into "special" PID
+         * of 1 behind more loosely related process.
+         */
+        int ret = pcmk__pid_active(child->pid, child->name);
+
+        if (ipc_pid && ((ret != pcmk_rc_ok)
                         || ipc_pid == PCMK__SPECIAL_PID
-                        || crm_pid_active(ipc_pid, child->name) == 1)) {
-            if (ret == 1) {
-                /* assume there's no forking-while-retaining-IPC-socket
-                   involved in the "children's" lifecycle, hence that the
-                   tracking got out of sync purely because of some external
-                   (esotheric?) forces (user initiated process "refresh" by
-                   force? or intentionally racing on start-up, even?), and
-                   that switching over to this other detected, authentic
-                   instance with an IPC already in possession is a better
-                   trade-off than "neutralizing" it first so as to give
-                   either the original or possibly a new to-be-spawned
-                   daemon process a leeway for operation, which would
-                   otherwise have to be carried out */
+                        || (pcmk__pid_active(ipc_pid,
+                                             child->name) == pcmk_rc_ok))) {
+            /* An unexpected (but authorized) process was detected at the IPC
+             * endpoint, and either it is active, or the child we're tracking is
+             * not.
+             */
+
+            if (ret == pcmk_rc_ok) {
+                /* The child we're tracking is active. Kill it, and adopt the
+                 * detected process. This assumes that our children don't fork
+                 * (thus getting a different PID owning the IPC), but rather the
+                 * tracking got out of sync because of some means external to
+                 * Pacemaker, and adopting the detected process is better than
+                 * killing it and possibly having to spawn a new child.
+                 */
                 /* not possessing IPC, afterall (what about corosync CPG?) */
                 stop_child(child, SIGKILL);
-            } else {
-                ret = 1;
             }
+            rc = pcmk_rc_ok;
             child->pid = ipc_pid;
-        } else if (ret == 1) {
-            ret = 2;  /* up per PID, but not per IPC (still starting?) */
-        } else if (!child->pid && ret == -1) {
-            ret = 0;  /* correct -1 on FreeBSD from above back to 0 */
+        } else if (ret == pcmk_rc_ok) {
+            // Our tracked child's PID was found active, but not its IPC
+            rc = pcmk_rc_ipc_pid_only;
+        } else if ((child->pid == 0) && (ret == EINVAL)) {
+            // FreeBSD can return EINVAL
+            rc = pcmk_rc_ipc_unresponsive;
+        } else {
+            switch (ret) {
+                case EACCES:
+                    rc = pcmk_rc_ipc_unauthorized;
+                    break;
+                case ESRCH:
+                    rc = pcmk_rc_ipc_unresponsive;
+                    break;
+                default:
+                    rc = ret;
+                    break;
+            }
         }
     }
-
-    return ret;
+    return rc;
 }
 
 static gboolean
@@ -870,35 +924,26 @@ check_active_before_startup_processes(gpointer user_data)
             } else if (start_seq != pcmk_children[lpc].start_seq) {
                 continue;
             } else {
-                const char *name = pcmk_children[lpc].name;
-                int ret;
+                int rc = child_liveness(&pcmk_children[lpc]);
 
-                switch ((ret = pcmk_child_active(&pcmk_children[lpc]))) {
-                    case 1:
+                switch (rc) {
+                    case pcmk_rc_ok:
                         break;
-                    case 0:
-                    case 2:  /* this very case: it was OK once already */
+                    case pcmk_rc_ipc_unresponsive:
+                    case pcmk_rc_ipc_pid_only: // This case: it was previously OK
                         if (pcmk_children[lpc].respawn == TRUE) {
-                            crm_err("%s[%d] terminated%s", name,
-                                    PCMK__SPECIAL_PID_AS_0(pcmk_children[lpc].pid),
-                                    ret ? " as IPC server" : "");
+                            crm_err("%s[%lld] terminated%s", pcmk_children[lpc].name,
+                                    (long long) PCMK__SPECIAL_PID_AS_0(pcmk_children[lpc].pid),
+                                    (rc == pcmk_rc_ipc_pid_only)? " as IPC server" : "");
                         } else {
                             /* orderly shutdown */
-                            crm_notice("%s[%d] terminated%s", name,
-                                       PCMK__SPECIAL_PID_AS_0(pcmk_children[lpc].pid),
-                                       ret ? " as IPC server" : "");
+                            crm_notice("%s[%lld] terminated%s", pcmk_children[lpc].name,
+                                       (long long) PCMK__SPECIAL_PID_AS_0(pcmk_children[lpc].pid),
+                                       (rc == pcmk_rc_ipc_pid_only)? " as IPC server" : "");
                         }
                         pcmk_process_exit(&(pcmk_children[lpc]));
                         continue;
                     default:
-                        crm_crit("Unexpected value from pcmk_child_active:"
-                                 " %d (pid=%lld)", ret,
-                                 (long long) PCMK__SPECIAL_PID_AS_0(
-                                                 pcmk_children[lpc].pid));
-                        /* fall through */
-                    case -1:
-                    case -2:
-                        /* message(s) already emitted */
                         crm_exit(CRM_EX_FATAL);
                         break;  /* static analysis/noreturn */
                 }
@@ -931,9 +976,7 @@ check_active_before_startup_processes(gpointer user_data)
  * will appear or the unattainable process will disappear per the observable
  * IPC, respectively.
  *
- * \return 0 if no such "child" process found, positive number X when X
- *         "children" detected, -1 on an internal error, -2 when any
- *         would-be-used IPC is blocked with an unauthorized process
+ * \return Standard Pacemaker return code
  *
  * \note Since this gets run at the very start, \c respawn_count fields
  *       for particular children get temporarily overloaded with "rounds
@@ -946,33 +989,39 @@ check_active_before_startup_processes(gpointer user_data)
 static int
 find_and_track_existing_processes(void)
 {
-    unsigned tracking = 0U;
+    bool tracking = false;
     bool wait_in_progress;
-    int cur;
+    int rc;
     size_t i, rounds;
 
     for (rounds = 1; rounds <= WAIT_TRIES; rounds++) {
         wait_in_progress = false;
         for (i = 0; i < SIZEOF(pcmk_children); i++) {
-            if (!pcmk_children[i].endpoint
-                    || pcmk_children[i].respawn_count < 0
-                    || !(cur = pcmk_child_active(&pcmk_children[i]))) {
-                /* as a speculation, don't give up in the context of
-                   pcmk_child_active check if there are more rounds to
-                   come for other reasons, but don't artificially wait just
-                   because of this, since we would preferably start ASAP */
+
+            if ((pcmk_children[i].endpoint == NULL)
+                || (pcmk_children[i].respawn_count < 0)) {
                 continue;
             }
+
+            rc = child_liveness(&pcmk_children[i]);
+            if (rc == pcmk_rc_ipc_unresponsive) {
+                /* As a speculation, don't give up if there are more rounds to
+                 * come for other reasons, but don't artificially wait just
+                 * because of this, since we would preferably start ASAP.
+                 */
+                continue;
+            }
+
             pcmk_children[i].respawn_count = rounds;
-            switch (cur) {
-                case 1:
+            switch (rc) {
+                case pcmk_rc_ok:
                     if (pcmk_children[i].pid == PCMK__SPECIAL_PID) {
                         if (crm_is_true(getenv("PCMK_fail_fast"))) {
                             crm_crit("Cannot reliably track pre-existing"
                                      " authentic process behind %s IPC on this"
                                      " platform and PCMK_fail_fast requested",
                                      pcmk_children[i].endpoint);
-                            return -1;
+                            return EOPNOTSUPP;
                         } else if (pcmk_children[i].respawn_count == WAIT_TRIES) {
                             crm_notice("Assuming pre-existing authentic, though"
                                        " on this platform untrackable, process"
@@ -1008,16 +1057,16 @@ find_and_track_existing_processes(void)
                                                pcmk_children[i].pid));
                     pcmk_children[i].respawn_count = -1;  /* 0~keep watching */
                     pcmk_children[i].active_before_startup = TRUE;
-                    tracking++;
+                    tracking = true;
                     break;
-                case 2:
+                case pcmk_rc_ipc_pid_only:
                     if (pcmk_children[i].respawn_count == WAIT_TRIES) {
                         crm_crit("%s IPC end-point for existing authentic"
                                  " process %lld did not (re)appear",
                                  pcmk_children[i].endpoint,
                                  (long long) PCMK__SPECIAL_PID_AS_0(
                                                  pcmk_children[i].pid));
-                        return -1;
+                        return rc;
                     }
                     wait_in_progress = true;
                     crm_warn("Cannot find %s IPC end-point for existing"
@@ -1028,12 +1077,10 @@ find_and_track_existing_processes(void)
                                              pcmk_children[i].pid),
                              WAIT_TRIES - pcmk_children[i].respawn_count);
                     continue;
-                case -1:
-                case -2:
-                    return cur;  /* messages already emitted */
                 default:
-                    crm_crit("Unexpected condition"CRM_XS"cur=%d", cur);
-                    return -1;  /* unexpected condition */
+                    crm_crit("Checked liveness of %s: %s " CRM_XS " rc=%d",
+                             pcmk_children[i].name, pcmk_rc_str(rc), rc);
+                    return rc;
             }
         }
         if (!wait_in_progress) {
@@ -1049,7 +1096,7 @@ find_and_track_existing_processes(void)
         g_timeout_add_seconds(PCMK_PROCESS_CHECK_INTERVAL,
                               check_active_before_startup_processes, NULL);
     }
-    return (tracking > INT_MAX) ? INT_MAX : tracking;
+    return pcmk_rc_ok;
 }
 
 static void
@@ -1062,7 +1109,7 @@ init_children_processes(void)
     for (start_seq = 1; start_seq < max; start_seq++) {
         /* don't start anything with start_seq < 1 */
         for (lpc = 0; lpc < max; lpc++) {
-            if (pcmk_children[lpc].pid) {
+            if (pcmk_children[lpc].pid != 0) {
                 /* we are already tracking it */
                 continue;
             }
@@ -1184,12 +1231,14 @@ main(int argc, char **argv)
     static crm_cluster_t cluster;
 
     crm_log_preinit(NULL, argc, argv);
-    crm_set_options(NULL, "mode [options]", long_options, "Start/Stop Pacemaker\n");
+    pcmk__set_cli_options(NULL, "[options]", long_options,
+                          "primary Pacemaker daemon that launches and "
+                          "monitors all subsidiary Pacemaker daemons");
     mainloop_add_signal(SIGHUP, pcmk_ignore);
     mainloop_add_signal(SIGQUIT, pcmk_sigquit);
 
     while (1) {
-        flag = crm_get_option(argc, argv, &option_index);
+        flag = pcmk__next_cli_option(argc, argv, &option_index, NULL);
         if (flag == -1)
             break;
 
@@ -1204,11 +1253,11 @@ main(int argc, char **argv)
                 pid_file = optarg;
                 break;
             case 's':
-                set_daemon_option("node_start_state", "standby");
+                pcmk__set_env_option("node_start_state", "standby");
                 break;
             case '$':
             case '?':
-                crm_help(flag, CRM_EX_OK);
+                pcmk__cli_help(flag, CRM_EX_OK);
                 break;
             case 'S':
                 shutdown = TRUE;
@@ -1231,13 +1280,13 @@ main(int argc, char **argv)
         printf("\n");
     }
     if (argerr) {
-        crm_help('?', CRM_EX_USAGE);
+        pcmk__cli_help('?', CRM_EX_USAGE);
     }
 
 
     setenv("LC_ALL", "C", 1);
 
-    set_daemon_option("mcp", "true");
+    pcmk__set_env_option("mcp", "true");
 
     crm_log_init(NULL, LOG_INFO, TRUE, FALSE, argc, argv, FALSE);
 
@@ -1278,7 +1327,7 @@ main(int argc, char **argv)
 
     // OCF shell functions and cluster-glue need facility under different name
     {
-        const char *facility = daemon_option("logfacility");
+        const char *facility = pcmk__env_option("logfacility");
 
         if (facility && safe_str_neq(facility, "none")) {
             setenv("HA_LOGFACILITY", facility, 1);
@@ -1309,7 +1358,7 @@ main(int argc, char **argv)
         }
     }
 
-    if (crm_user_lookup(CRM_DAEMON_USER, &pcmk_uid, &pcmk_gid) < 0) {
+    if (pcmk_daemon_user(&pcmk_uid, &pcmk_gid) < 0) {
         crm_err("Cluster user %s does not exist, aborting Pacemaker startup", CRM_DAEMON_USER);
         crm_exit(CRM_EX_NOUSER);
     }
@@ -1362,14 +1411,12 @@ main(int argc, char **argv)
     }
 
     switch (find_and_track_existing_processes()) {
-        case -1:
-            crm_crit("Internal fatality, see the log");
-            crm_exit(CRM_EX_FATAL);
-        case -2:
-            crm_crit("Blocked by foreign process, kill the offender");
+        case pcmk_rc_ok:
+            break;
+        case pcmk_rc_ipc_unauthorized:
             crm_exit(CRM_EX_CANTCREAT);
         default:
-            break;
+            crm_exit(CRM_EX_FATAL);
     };
 
     cluster.destroy = mcp_cpg_destroy;
