@@ -1,23 +1,23 @@
-/* 
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
- * 
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- * 
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+/*
+ * Copyright 2004-2019 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
+ *
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
+
 #ifndef CIB_UTIL__H
 #  define CIB_UTIL__H
 
+#include <glib.h>               // gboolean
+#include <libxml/tree.h>        // xmlNode
+#include <crm/cib/cib_types.h>  // cib_t
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* Utility functions */
 const char *get_object_path(const char *object_type);
 const char *get_object_parent(const char *object_type);
@@ -60,11 +60,14 @@ int query_node_uname(cib_t * the_cib, const char *uuid, char **uname);
 
 int set_standby(cib_t * the_cib, const char *uuid, const char *scope, const char *standby_value);
 
-xmlNode *get_cib_copy(cib_t * cib);
 xmlNode *cib_get_generation(cib_t * cib);
 
 void cib_metadata(void);
 const char *cib_pref(GHashTable * options, const char *name);
 int cib_apply_patch_event(xmlNode * event, xmlNode * input, xmlNode ** output, int level);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
