@@ -64,7 +64,7 @@ enum crm_proc_flag {
  * \return Process flag if detectable, otherwise 0
  */
 static inline uint32_t
-crm_get_cluster_proc()
+crm_get_cluster_proc(void)
 {
     switch (get_cluster_type()) {
         case pcmk_cluster_corosync:
@@ -317,7 +317,7 @@ gboolean init_cs_connection_once(crm_cluster_t * cluster);
 crm_node_t *crm_update_peer_proc(const char *source, crm_node_t * peer,
                                  uint32_t flag, const char *status);
 crm_node_t *crm_update_peer_state(const char *source, crm_node_t * node,
-                                  const char *state, int membership);
+                                  const char *state, uint64_t membership);
 
 void crm_update_peer_uname(crm_node_t *node, const char *uname);
 void crm_update_peer_expected(const char *source, crm_node_t * node, const char *expected);
