@@ -1,5 +1,7 @@
 /*
- * Copyright 2012-2018 David Vossel <davidvossel@gmail.com>
+ * Copyright 2012-2020 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -22,9 +24,10 @@
 
 #include <crm/crm.h>
 #include <crm/lrmd.h>
+#include <crm/lrmd_internal.h>
 #include <crm/services.h>
 #include <crm/common/mainloop.h>
-#include <crm/common/ipcs.h>
+#include <crm/common/ipcs_internal.h>
 #include <crm/common/remote_internal.h>
 #include <crm/msg_xml.h>
 
@@ -59,19 +62,18 @@ gnutls_psk_client_credentials_t psk_cred_s;
 int lrmd_tls_set_key(gnutls_datum_t * key);
 static void lrmd_tls_disconnect(lrmd_t * lrmd);
 static int global_remote_msg_id = 0;
-int lrmd_tls_send_msg(crm_remote_t * session, xmlNode * msg, uint32_t id, const char *msg_type);
 static void lrmd_tls_connection_destroy(gpointer userdata);
 #endif
 
 typedef struct lrmd_private_s {
-    enum client_type type;
+    enum pcmk__client_type type;
     char *token;
     mainloop_io_t *source;
 
     /* IPC parameters */
     crm_ipc_t *ipc;
 
-    crm_remote_t *remote;
+    pcmk__remote_t *remote;
 
     /* Extra TLS parameters */
     char *remote_nodename;
@@ -175,6 +177,36 @@ lrmd_key_value_freeall(lrmd_key_value_t * head)
     }
 }
 
+/*!
+ * Create a new lrmd_event_data_t object
+ *
+ * \param[in] rsc_id       ID of resource involved in event
+ * \param[in] task         Action name
+ * \param[in] interval_ms  Action interval
+ *
+ * \return Newly allocated and initialized lrmd_event_data_t
+ * \note This functions asserts on memory errors, so the return value is
+ *       guaranteed to be non-NULL. The caller is responsible for freeing the
+ *       result with lrmd_free_event().
+ */
+lrmd_event_data_t *
+lrmd_new_event(const char *rsc_id, const char *task, guint interval_ms)
+{
+    lrmd_event_data_t *event = calloc(1, sizeof(lrmd_event_data_t));
+
+    CRM_ASSERT(event != NULL);
+    if (rsc_id != NULL) {
+        event->rsc_id = strdup(rsc_id);
+        CRM_ASSERT(event->rsc_id != NULL);
+    }
+    if (task != NULL) {
+        event->op_type = strdup(task);
+        CRM_ASSERT(event->op_type != NULL);
+    }
+    event->interval_ms = interval_ms;
+    return event;
+}
+
 lrmd_event_data_t *
 lrmd_copy_event(lrmd_event_data_t * event)
 {
@@ -332,8 +364,7 @@ lrmd_tls_dispatch(gpointer userdata)
     lrmd_t *lrmd = userdata;
     lrmd_private_t *native = lrmd->lrmd_private;
     xmlNode *xml = NULL;
-    int rc = 0;
-    int disconnected = 0;
+    int rc = pcmk_rc_ok;
 
     if (lrmd_tls_connected(lrmd) == FALSE) {
         crm_trace("TLS dispatch triggered after disconnect");
@@ -356,15 +387,18 @@ lrmd_tls_dispatch(gpointer userdata)
     }
 
     /* Next read the current buffer and see if there are any messages to handle. */
-    rc = crm_remote_ready(native->remote, 0);
-    if (rc == 0) {
-        /* nothing to read, see if any full messages are already in buffer. */
-        xml = crm_remote_parse_buffer(native->remote);
-    } else if (rc < 0) {
-        disconnected = 1;
-    } else {
-        crm_remote_recv(native->remote, -1, &disconnected);
-        xml = crm_remote_parse_buffer(native->remote);
+    switch (pcmk__remote_ready(native->remote, 0)) {
+        case pcmk_rc_ok:
+            rc = pcmk__read_remote_message(native->remote, -1);
+            xml = pcmk__remote_message_xml(native->remote);
+            break;
+        case ETIME:
+            // Nothing to read, check if a full message is already in buffer
+            xml = pcmk__remote_message_xml(native->remote);
+            break;
+        default:
+            rc = ENOTCONN;
+            break;
     }
     while (xml) {
         const char *msg_type = crm_element_value(xml, F_LRMD_REMOTE_MSG_TYPE);
@@ -381,10 +415,10 @@ lrmd_tls_dispatch(gpointer userdata)
             }
         }
         free_xml(xml);
-        xml = crm_remote_parse_buffer(native->remote);
+        xml = pcmk__remote_message_xml(native->remote);
     }
 
-    if (disconnected) {
+    if (rc == ENOTCONN) {
         crm_info("Lost %s executor connection while reading data",
                  (native->remote_nodename? native->remote_nodename : "local"));
         lrmd_tls_disconnect(lrmd);
@@ -401,16 +435,25 @@ lrmd_poll(lrmd_t * lrmd, int timeout)
     lrmd_private_t *native = lrmd->lrmd_private;
 
     switch (native->type) {
-        case CRM_CLIENT_IPC:
+        case PCMK__CLIENT_IPC:
             return crm_ipc_ready(native->ipc);
 
 #ifdef HAVE_GNUTLS_GNUTLS_H
-        case CRM_CLIENT_TLS:
+        case PCMK__CLIENT_TLS:
             if (native->pending_notify) {
                 return 1;
-            }
+            } else {
+                int rc = pcmk__remote_ready(native->remote, 0);
 
-            return crm_remote_ready(native->remote, 0);
+                switch (rc) {
+                    case pcmk_rc_ok:
+                        return 1;
+                    case ETIME:
+                        return 0;
+                    default:
+                        return pcmk_rc2legacy(rc);
+                }
+            }
 #endif
         default:
             crm_err("Unsupported connection type: %d", native->type);
@@ -429,7 +472,7 @@ lrmd_dispatch(lrmd_t * lrmd)
 
     private = lrmd->lrmd_private;
     switch (private->type) {
-        case CRM_CLIENT_IPC:
+        case PCMK__CLIENT_IPC:
             while (crm_ipc_ready(private->ipc)) {
                 if (crm_ipc_read(private->ipc) > 0) {
                     const char *msg = crm_ipc_buffer(private->ipc);
@@ -439,7 +482,7 @@ lrmd_dispatch(lrmd_t * lrmd)
             }
             break;
 #ifdef HAVE_GNUTLS_GNUTLS_H
-        case CRM_CLIENT_TLS:
+        case PCMK__CLIENT_TLS:
             lrmd_tls_dispatch(lrmd);
             break;
 #endif
@@ -546,12 +589,14 @@ lrmd_tls_connection_destroy(gpointer userdata)
     return;
 }
 
+// \return Standard Pacemaker return code
 int
-lrmd_tls_send_msg(crm_remote_t * session, xmlNode * msg, uint32_t id, const char *msg_type)
+lrmd_tls_send_msg(pcmk__remote_t *session, xmlNode *msg, uint32_t id,
+                  const char *msg_type)
 {
     crm_xml_add_int(msg, F_LRMD_REMOTE_MSG_ID, id);
     crm_xml_add(msg, F_LRMD_REMOTE_MSG_TYPE, msg_type);
-    return crm_remote_send(session, msg);
+    return pcmk__remote_send_xml(session, msg);
 }
 
 static xmlNode *
@@ -572,7 +617,7 @@ lrmd_tls_recv_reply(lrmd_t * lrmd, int total_timeout, int expected_reply_id, int
 
     while (!xml) {
 
-        xml = crm_remote_parse_buffer(native->remote);
+        xml = pcmk__remote_message_xml(native->remote);
         if (!xml) {
             /* read some more off the tls buffer if we still have time left. */
             if (remaining_timeout) {
@@ -586,8 +631,13 @@ lrmd_tls_recv_reply(lrmd_t * lrmd, int total_timeout, int expected_reply_id, int
                 return NULL;
             }
 
-            crm_remote_recv(native->remote, remaining_timeout, disconnected);
-            xml = crm_remote_parse_buffer(native->remote);
+            if (pcmk__read_remote_message(native->remote,
+                                          remaining_timeout) == ENOTCONN) {
+                *disconnected = TRUE;
+            } else {
+                *disconnected = FALSE;
+            }
+            xml = pcmk__remote_message_xml(native->remote);
             if (!xml) {
                 crm_err("Unable to receive expected reply, disconnecting.");
                 *disconnected = TRUE;
@@ -650,8 +700,9 @@ lrmd_tls_send(lrmd_t * lrmd, xmlNode * msg)
     }
 
     rc = lrmd_tls_send_msg(native->remote, msg, global_remote_msg_id, "request");
-    if (rc <= 0) {
-        crm_err("Disconnecting because TLS message could not be sent to Pacemaker Remote");
+    if (rc != pcmk_rc_ok) {
+        crm_err("Disconnecting because TLS message could not be sent to "
+                "Pacemaker Remote: %s", pcmk_rc_str(rc));
         lrmd_tls_disconnect(lrmd);
         return -ENOTCONN;
     }
@@ -704,11 +755,11 @@ lrmd_send_xml(lrmd_t * lrmd, xmlNode * msg, int timeout, xmlNode ** reply)
     lrmd_private_t *native = lrmd->lrmd_private;
 
     switch (native->type) {
-        case CRM_CLIENT_IPC:
+        case PCMK__CLIENT_IPC:
             rc = crm_ipc_send(native->ipc, msg, crm_ipc_client_response, timeout, reply);
             break;
 #ifdef HAVE_GNUTLS_GNUTLS_H
-        case CRM_CLIENT_TLS:
+        case PCMK__CLIENT_TLS:
             rc = lrmd_tls_send_recv(lrmd, msg, timeout, reply);
             break;
 #endif
@@ -726,11 +777,11 @@ lrmd_send_xml_no_reply(lrmd_t * lrmd, xmlNode * msg)
     lrmd_private_t *native = lrmd->lrmd_private;
 
     switch (native->type) {
-        case CRM_CLIENT_IPC:
+        case PCMK__CLIENT_IPC:
             rc = crm_ipc_send(native->ipc, msg, crm_ipc_flags_none, 0, NULL);
             break;
 #ifdef HAVE_GNUTLS_GNUTLS_H
-        case CRM_CLIENT_TLS:
+        case PCMK__CLIENT_TLS:
             rc = lrmd_tls_send(lrmd, msg);
             if (rc == pcmk_ok) {
                 /* we don't want to wait around for the reply, but
@@ -753,11 +804,11 @@ lrmd_api_is_connected(lrmd_t * lrmd)
     lrmd_private_t *native = lrmd->lrmd_private;
 
     switch (native->type) {
-        case CRM_CLIENT_IPC:
+        case PCMK__CLIENT_IPC:
             return crm_ipc_connected(native->ipc);
             break;
 #ifdef HAVE_GNUTLS_GNUTLS_H
-        case CRM_CLIENT_TLS:
+        case PCMK__CLIENT_TLS:
             return lrmd_tls_connected(lrmd);
             break;
 #endif
@@ -864,7 +915,8 @@ lrmd_api_poke_connection(lrmd_t * lrmd)
     xmlNode *data = create_xml_node(NULL, F_LRMD_RSC);
 
     crm_xml_add(data, F_LRMD_ORIGIN, __FUNCTION__);
-    rc = lrmd_send_command(lrmd, LRMD_OP_POKE, data, NULL, 0, 0, native->type == CRM_CLIENT_IPC ? TRUE : FALSE);
+    rc = lrmd_send_command(lrmd, LRMD_OP_POKE, data, NULL, 0, 0,
+                           (native->type == PCMK__CLIENT_IPC));
     free_xml(data);
 
     return rc < 0 ? rc : pcmk_ok;
@@ -883,7 +935,8 @@ remote_proxy_check(lrmd_t * lrmd, GHashTable *hash)
     value = g_hash_table_lookup(hash, "stonith-watchdog-timeout");
     crm_xml_add(data, F_LRMD_WATCHDOG, value);
 
-    rc = lrmd_send_command(lrmd, LRMD_OP_CHECK, data, NULL, 0, 0, native->type == CRM_CLIENT_IPC ? TRUE : FALSE);
+    rc = lrmd_send_command(lrmd, LRMD_OP_CHECK, data, NULL, 0, 0,
+                           (native->type == PCMK__CLIENT_IPC));
     free_xml(data);
 
     return rc < 0 ? rc : pcmk_ok;
@@ -958,7 +1011,7 @@ lrmd_ipc_connect(lrmd_t * lrmd, int *fd)
     int rc = pcmk_ok;
     lrmd_private_t *native = lrmd->lrmd_private;
 
-    static struct ipc_client_callbacks lrmd_callbacks = {
+    struct ipc_client_callbacks lrmd_callbacks = {
         .dispatch = lrmd_ipc_dispatch,
         .destroy = lrmd_ipc_connection_destroy
     };
@@ -1119,8 +1172,14 @@ report_async_connection_result(lrmd_t * lrmd, int rc)
 }
 
 #ifdef HAVE_GNUTLS_GNUTLS_H
+static inline int
+lrmd__tls_client_handshake(pcmk__remote_t *remote)
+{
+    return pcmk__tls_client_handshake(remote, LRMD_CLIENT_HANDSHAKE_TIMEOUT);
+}
+
 static void
-lrmd_tcp_connect_cb(void *userdata, int sock)
+lrmd_tcp_connect_cb(void *userdata, int rc, int sock)
 {
     lrmd_t *lrmd = userdata;
     lrmd_private_t *native = lrmd->lrmd_private;
@@ -1129,16 +1188,16 @@ lrmd_tcp_connect_cb(void *userdata, int sock)
         .dispatch = lrmd_tls_dispatch,
         .destroy = lrmd_tls_connection_destroy,
     };
-    int rc = sock;
     gnutls_datum_t psk_key = { NULL, 0 };
 
     native->async_timer = 0;
 
-    if (rc < 0) {
+    if (rc != pcmk_rc_ok) {
         lrmd_tls_connection_destroy(lrmd);
-        crm_info("Could not connect to Pacemaker Remote at %s:%d",
-                 native->server, native->port);
-        report_async_connection_result(lrmd, rc);
+        crm_info("Could not connect to Pacemaker Remote at %s:%d: %s "
+                 CRM_XS " rc=%d",
+                 native->server, native->port, pcmk_rc_str(rc), rc);
+        report_async_connection_result(lrmd, pcmk_rc2legacy(rc));
         return;
     }
 
@@ -1170,7 +1229,7 @@ lrmd_tcp_connect_cb(void *userdata, int sock)
         return;
     }
 
-    if (crm_initiate_client_tls_handshake(native->remote, LRMD_CLIENT_HANDSHAKE_TIMEOUT) != 0) {
+    if (lrmd__tls_client_handshake(native->remote) != pcmk_rc_ok) {
         crm_warn("Disconnecting after TLS handshake with Pacemaker Remote server %s:%d failed",
                  native->server, native->port);
         gnutls_deinit(*native->remote->tls_session);
@@ -1201,17 +1260,20 @@ lrmd_tcp_connect_cb(void *userdata, int sock)
 static int
 lrmd_tls_connect_async(lrmd_t * lrmd, int timeout /*ms */ )
 {
-    int sock = 0;
+    int rc;
     int timer_id = 0;
     lrmd_private_t *native = lrmd->lrmd_private;
 
     lrmd_gnutls_global_init();
-    sock = crm_remote_tcp_connect_async(native->server, native->port, timeout,
-                                        &timer_id, lrmd, lrmd_tcp_connect_cb);
-    if (sock < 0) {
-        return sock;
+    native->sock = -1;
+    rc = pcmk__connect_remote(native->server, native->port, timeout, &timer_id,
+                              &(native->sock), lrmd, lrmd_tcp_connect_cb);
+    if (rc != pcmk_rc_ok) {
+        crm_warn("Pacemaker Remote connection to %s:%s failed: %s "
+                 CRM_XS " rc=%d",
+                 native->server, native->port, pcmk_rc_str(rc), rc);
+        return -1;
     }
-    native->sock = sock;
     native->async_timer = timer_id;
     return pcmk_ok;
 }
@@ -1226,20 +1288,21 @@ lrmd_tls_connect(lrmd_t * lrmd, int *fd)
     int rc;
 
     lrmd_private_t *native = lrmd->lrmd_private;
-    int sock;
     gnutls_datum_t psk_key = { NULL, 0 };
 
     lrmd_gnutls_global_init();
 
-    sock = crm_remote_tcp_connect(native->server, native->port);
-    if (sock < 0) {
-        crm_warn("Could not establish Pacemaker Remote connection to %s", native->server);
+    native->sock = -1;
+    rc = pcmk__connect_remote(native->server, native->port, 0, NULL,
+                              &(native->sock), NULL, NULL);
+    if (rc != pcmk_rc_ok) {
+        crm_warn("Pacemaker Remote connection to %s:%s failed: %s "
+                 CRM_XS " rc=%d",
+                 native->server, native->port, pcmk_rc_str(rc), rc);
         lrmd_tls_connection_destroy(lrmd);
         return -ENOTCONN;
     }
 
-    native->sock = sock;
-
     rc = lrmd_tls_set_key(&psk_key);
     if (rc < 0) {
         lrmd_tls_connection_destroy(lrmd);
@@ -1250,7 +1313,7 @@ lrmd_tls_connect(lrmd_t * lrmd, int *fd)
     gnutls_psk_set_client_credentials(native->psk_cred_c, DEFAULT_REMOTE_USERNAME, &psk_key, GNUTLS_PSK_KEY_RAW);
     gnutls_free(psk_key.data);
 
-    native->remote->tls_session = pcmk__new_tls_session(sock, GNUTLS_CLIENT,
+    native->remote->tls_session = pcmk__new_tls_session(native->sock, GNUTLS_CLIENT,
                                                         GNUTLS_CRD_PSK,
                                                         native->psk_cred_c);
     if (native->remote->tls_session == NULL) {
@@ -1258,7 +1321,7 @@ lrmd_tls_connect(lrmd_t * lrmd, int *fd)
         return -EPROTO;
     }
 
-    if (crm_initiate_client_tls_handshake(native->remote, LRMD_CLIENT_HANDSHAKE_TIMEOUT) != 0) {
+    if (lrmd__tls_client_handshake(native->remote) != pcmk_rc_ok) {
         crm_err("Session creation for %s:%d failed", native->server, native->port);
         gnutls_deinit(*native->remote->tls_session);
         gnutls_free(native->remote->tls_session);
@@ -1271,7 +1334,7 @@ lrmd_tls_connect(lrmd_t * lrmd, int *fd)
              native->port);
 
     if (fd) {
-        *fd = sock;
+        *fd = native->sock;
     } else {
         char *name = crm_strdup_printf("pacemaker-remote-%s:%d",
                                        native->server, native->port);
@@ -1292,11 +1355,11 @@ lrmd_api_connect(lrmd_t * lrmd, const char *name, int *fd)
     lrmd_private_t *native = lrmd->lrmd_private;
 
     switch (native->type) {
-        case CRM_CLIENT_IPC:
+        case PCMK__CLIENT_IPC:
             rc = lrmd_ipc_connect(lrmd, fd);
             break;
 #ifdef HAVE_GNUTLS_GNUTLS_H
-        case CRM_CLIENT_TLS:
+        case PCMK__CLIENT_TLS:
             rc = lrmd_tls_connect(lrmd, fd);
             break;
 #endif
@@ -1320,7 +1383,7 @@ lrmd_api_connect_async(lrmd_t * lrmd, const char *name, int timeout)
     CRM_CHECK(native && native->callback, return -1);
 
     switch (native->type) {
-        case CRM_CLIENT_IPC:
+        case PCMK__CLIENT_IPC:
             /* fake async connection with ipc.  it should be fast
              * enough that we gain very little from async */
             rc = lrmd_api_connect(lrmd, name, NULL);
@@ -1329,7 +1392,7 @@ lrmd_api_connect_async(lrmd_t * lrmd, const char *name, int timeout)
             }
             break;
 #ifdef HAVE_GNUTLS_GNUTLS_H
-        case CRM_CLIENT_TLS:
+        case PCMK__CLIENT_TLS:
             rc = lrmd_tls_connect_async(lrmd, timeout);
             if (rc) {
                 /* connection failed, report rc now */
@@ -1406,14 +1469,14 @@ lrmd_api_disconnect(lrmd_t * lrmd)
     lrmd_private_t *native = lrmd->lrmd_private;
 
     crm_info("Disconnecting %s %s executor connection",
-             crm_client_type_text(native->type),
+             pcmk__client_type_str(native->type),
              (native->remote_nodename? native->remote_nodename : "local"));
     switch (native->type) {
-        case CRM_CLIENT_IPC:
+        case PCMK__CLIENT_IPC:
             lrmd_ipc_disconnect(lrmd);
             break;
 #ifdef HAVE_GNUTLS_GNUTLS_H
-        case CRM_CLIENT_TLS:
+        case PCMK__CLIENT_TLS:
             lrmd_tls_disconnect(lrmd);
             break;
 #endif
@@ -1960,10 +2023,10 @@ lrmd_api_new(void)
 
     new_lrmd = calloc(1, sizeof(lrmd_t));
     pvt = calloc(1, sizeof(lrmd_private_t));
-    pvt->remote = calloc(1, sizeof(crm_remote_t));
+    pvt->remote = calloc(1, sizeof(pcmk__remote_t));
     new_lrmd->cmds = calloc(1, sizeof(lrmd_api_operations_t));
 
-    pvt->type = CRM_CLIENT_IPC;
+    pvt->type = PCMK__CLIENT_IPC;
     new_lrmd->lrmd_private = pvt;
 
     new_lrmd->cmds->connect = lrmd_api_connect;
@@ -2000,7 +2063,7 @@ lrmd_remote_api_new(const char *nodename, const char *server, int port)
         return NULL;
     }
 
-    native->type = CRM_CLIENT_TLS;
+    native->type = PCMK__CLIENT_TLS;
     native->remote_nodename = nodename ? strdup(nodename) : strdup(server);
     native->server = server ? strdup(server) : strdup(nodename);
     native->port = port;
