@@ -1,5 +1,7 @@
 /*
- * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2020 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU General Public License version 2
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
@@ -14,11 +16,12 @@
 #define VARIANT_GROUP 1
 #include <lib/pengine/variant.h>
 
-node_t *
-group_color(resource_t * rsc, node_t * prefer, pe_working_set_t * data_set)
+pe_node_t *
+pcmk__group_allocate(pe_resource_t *rsc, pe_node_t *prefer,
+                     pe_working_set_t *data_set)
 {
-    node_t *node = NULL;
-    node_t *group_node = NULL;
+    pe_node_t *node = NULL;
+    pe_node_t *group_node = NULL;
     GListPtr gIter = NULL;
     group_variant_data_t *group_data = NULL;
 
@@ -27,14 +30,13 @@ group_color(resource_t * rsc, node_t * prefer, pe_working_set_t * data_set)
     if (is_not_set(rsc->flags, pe_rsc_provisional)) {
         return rsc->allocated_to;
     }
-    pe_rsc_trace(rsc, "Processing %s", rsc->id);
     if (is_set(rsc->flags, pe_rsc_allocating)) {
         pe_rsc_debug(rsc, "Dependency loop detected involving %s", rsc->id);
         return NULL;
     }
 
     if (group_data->first_child == NULL) {
-        /* nothign to allocate */
+        // Nothing to allocate
         clear_bit(rsc->flags, pe_rsc_provisional);
         return NULL;
     }
@@ -50,13 +52,14 @@ group_color(resource_t * rsc, node_t * prefer, pe_working_set_t * data_set)
         g_list_concat(group_data->last_child->rsc_cons_lhs, rsc->rsc_cons_lhs);
     rsc->rsc_cons_lhs = NULL;
 
-    dump_node_scores(show_scores ? 0 : scores_log_level, rsc, __FUNCTION__,
-                     rsc->allowed_nodes);
+    pe__show_node_weights(!show_scores, rsc, __FUNCTION__, rsc->allowed_nodes);
 
     gIter = rsc->children;
     for (; gIter != NULL; gIter = gIter->next) {
-        resource_t *child_rsc = (resource_t *) gIter->data;
+        pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
 
+        pe_rsc_trace(rsc, "Allocating group %s member %s",
+                     rsc->id, child_rsc->id);
         node = child_rsc->cmds->allocate(child_rsc, prefer, data_set);
         if (group_node == NULL) {
             group_node = node;
@@ -73,19 +76,19 @@ group_color(resource_t * rsc, node_t * prefer, pe_working_set_t * data_set)
     return NULL;
 }
 
-void group_update_pseudo_status(resource_t * parent, resource_t * child);
+void group_update_pseudo_status(pe_resource_t * parent, pe_resource_t * child);
 
 void
-group_create_actions(resource_t * rsc, pe_working_set_t * data_set)
+group_create_actions(pe_resource_t * rsc, pe_working_set_t * data_set)
 {
-    action_t *op = NULL;
+    pe_action_t *op = NULL;
     const char *value = NULL;
     GListPtr gIter = rsc->children;
 
     pe_rsc_trace(rsc, "Creating actions for %s", rsc->id);
 
     for (; gIter != NULL; gIter = gIter->next) {
-        resource_t *child_rsc = (resource_t *) gIter->data;
+        pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
 
         child_rsc->cmds->create_actions(child_rsc, data_set);
         group_update_pseudo_status(rsc, child_rsc);
@@ -124,7 +127,7 @@ group_create_actions(resource_t * rsc, pe_working_set_t * data_set)
 }
 
 void
-group_update_pseudo_status(resource_t * parent, resource_t * child)
+group_update_pseudo_status(pe_resource_t * parent, pe_resource_t * child)
 {
     GListPtr gIter = child->actions;
     group_variant_data_t *group_data = NULL;
@@ -141,7 +144,7 @@ group_update_pseudo_status(resource_t * parent, resource_t * child)
     }
 
     for (; gIter != NULL; gIter = gIter->next) {
-        action_t *action = (action_t *) gIter->data;
+        pe_action_t *action = (pe_action_t *) gIter->data;
 
         if (is_set(action->flags, pe_action_optional)) {
             continue;
@@ -159,12 +162,12 @@ group_update_pseudo_status(resource_t * parent, resource_t * child)
 }
 
 void
-group_internal_constraints(resource_t * rsc, pe_working_set_t * data_set)
+group_internal_constraints(pe_resource_t * rsc, pe_working_set_t * data_set)
 {
     GListPtr gIter = rsc->children;
-    resource_t *last_rsc = NULL;
-    resource_t *last_active = NULL;
-    resource_t *top = uber_parent(rsc);
+    pe_resource_t *last_rsc = NULL;
+    pe_resource_t *last_active = NULL;
+    pe_resource_t *top = uber_parent(rsc);
     group_variant_data_t *group_data = NULL;
 
     get_group_variant_data(group_data, rsc);
@@ -174,7 +177,7 @@ group_internal_constraints(resource_t * rsc, pe_working_set_t * data_set)
     new_rsc_order(rsc, RSC_STOP, rsc, RSC_STOPPED, pe_order_runnable_left, data_set);
 
     for (; gIter != NULL; gIter = gIter->next) {
-        resource_t *child_rsc = (resource_t *) gIter->data;
+        pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
         int stop = pe_order_none;
         int stopped = pe_order_implies_then_printed;
         int start = pe_order_implies_then | pe_order_runnable_left;
@@ -294,6 +297,9 @@ group_rsc_colocation_lh(pe_resource_t *rsc_lh, pe_resource_t *rsc_rh,
         pe_err("rsc_rh was NULL for %s", constraint->id);
         return;
     }
+    if (constraint->score == 0) {
+        return;
+    }
 
     gIter = rsc_lh->children;
     pe_rsc_trace(rsc_lh, "Processing constraints from %s", rsc_lh->id);
@@ -307,13 +313,14 @@ group_rsc_colocation_lh(pe_resource_t *rsc_lh, pe_resource_t *rsc_rh,
         return;
 
     } else if (constraint->score >= INFINITY) {
-        crm_config_err("%s: Cannot perform mandatory colocation"
-                       " between non-colocated group and %s", rsc_lh->id, rsc_rh->id);
+        pcmk__config_err("%s: Cannot perform mandatory colocation "
+                         "between non-colocated group and %s",
+                         rsc_lh->id, rsc_rh->id);
         return;
     }
 
     for (; gIter != NULL; gIter = gIter->next) {
-        resource_t *child_rsc = (resource_t *) gIter->data;
+        pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
 
         child_rsc->cmds->rsc_colocation_lh(child_rsc, rsc_rh, constraint,
                                            data_set);
@@ -331,6 +338,9 @@ group_rsc_colocation_rh(pe_resource_t *rsc_lh, pe_resource_t *rsc_rh,
     get_group_variant_data(group_data, rsc_rh);
     CRM_CHECK(rsc_lh->variant == pe_native, return);
 
+    if (constraint->score == 0) {
+        return;
+    }
     pe_rsc_trace(rsc_rh, "Processing RH %s of constraint %s (LH is %s)",
                  rsc_rh->id, constraint->id, rsc_lh->id);
 
@@ -355,13 +365,13 @@ group_rsc_colocation_rh(pe_resource_t *rsc_lh, pe_resource_t *rsc_rh,
         return;
 
     } else if (constraint->score >= INFINITY) {
-        crm_config_err("%s: Cannot perform mandatory colocation with"
-                       " non-colocated group: %s", rsc_lh->id, rsc_rh->id);
+        pcmk__config_err("%s: Cannot perform mandatory colocation with"
+                         " non-colocated group %s", rsc_lh->id, rsc_rh->id);
         return;
     }
 
     for (; gIter != NULL; gIter = gIter->next) {
-        resource_t *child_rsc = (resource_t *) gIter->data;
+        pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
 
         child_rsc->cmds->rsc_colocation_rh(rsc_lh, child_rsc, constraint,
                                            data_set);
@@ -369,16 +379,16 @@ group_rsc_colocation_rh(pe_resource_t *rsc_lh, pe_resource_t *rsc_rh,
 }
 
 enum pe_action_flags
-group_action_flags(action_t * action, node_t * node)
+group_action_flags(pe_action_t * action, pe_node_t * node)
 {
     GListPtr gIter = NULL;
     enum pe_action_flags flags = (pe_action_optional | pe_action_runnable | pe_action_pseudo);
 
     for (gIter = action->rsc->children; gIter != NULL; gIter = gIter->next) {
-        resource_t *child = (resource_t *) gIter->data;
+        pe_resource_t *child = (pe_resource_t *) gIter->data;
         enum action_tasks task = get_complex_task(child, action->task, TRUE);
         const char *task_s = task2text(task);
-        action_t *child_action = find_first_action(child->actions, NULL, task_s, node);
+        pe_action_t *child_action = find_first_action(child->actions, NULL, task_s, node);
 
         if (child_action) {
             enum pe_action_flags child_flags = child->cmds->action_flags(child_action, node);
@@ -422,8 +432,8 @@ group_update_actions(pe_action_t *first, pe_action_t *then, pe_node_t *node,
                                      data_set);
 
     for (; gIter != NULL; gIter = gIter->next) {
-        resource_t *child = (resource_t *) gIter->data;
-        action_t *child_action = find_first_action(child->actions, NULL, then->task, node);
+        pe_resource_t *child = (pe_resource_t *) gIter->data;
+        pe_action_t *child_action = find_first_action(child->actions, NULL, then->task, node);
 
         if (child_action) {
             changed |= child->cmds->update_actions(first, child_action, node,
@@ -440,7 +450,7 @@ group_rsc_location(pe_resource_t *rsc, pe__location_t *constraint)
 {
     GListPtr gIter = rsc->children;
     GListPtr saved = constraint->node_list_rh;
-    GListPtr zero = node_list_dup(constraint->node_list_rh, TRUE, FALSE);
+    GListPtr zero = pcmk__copy_node_list(constraint->node_list_rh, true);
     gboolean reset_scores = TRUE;
     group_variant_data_t *group_data = NULL;
 
@@ -451,7 +461,7 @@ group_rsc_location(pe_resource_t *rsc, pe__location_t *constraint)
     native_rsc_location(rsc, constraint);
 
     for (; gIter != NULL; gIter = gIter->next) {
-        resource_t *child_rsc = (resource_t *) gIter->data;
+        pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
 
         child_rsc->cmds->rsc_location(child_rsc, constraint);
         if (group_data->colocated && reset_scores) {
@@ -465,7 +475,7 @@ group_rsc_location(pe_resource_t *rsc, pe__location_t *constraint)
 }
 
 void
-group_expand(resource_t * rsc, pe_working_set_t * data_set)
+group_expand(pe_resource_t * rsc, pe_working_set_t * data_set)
 {
     CRM_CHECK(rsc != NULL, return);
 
@@ -473,15 +483,16 @@ group_expand(resource_t * rsc, pe_working_set_t * data_set)
     native_expand(rsc, data_set);
 
     for (GListPtr gIter = rsc->children; gIter != NULL; gIter = gIter->next) {
-        resource_t *child_rsc = (resource_t *) gIter->data;
+        pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
 
         child_rsc->cmds->expand(child_rsc, data_set);
     }
 }
 
 GHashTable *
-group_merge_weights(resource_t * rsc, const char *rhs, GHashTable * nodes, const char *attr,
-                    float factor, enum pe_weights flags)
+pcmk__group_merge_weights(pe_resource_t *rsc, const char *rhs,
+                          GHashTable *nodes, const char *attr, float factor,
+                          uint32_t flags)
 {
     GListPtr gIter = rsc->rsc_cons_lhs;
     group_variant_data_t *group_data = NULL;
@@ -502,9 +513,13 @@ group_merge_weights(resource_t * rsc, const char *rhs, GHashTable * nodes, const
     for (; gIter != NULL; gIter = gIter->next) {
         rsc_colocation_t *constraint = (rsc_colocation_t *) gIter->data;
 
-        nodes = native_merge_weights(constraint->rsc_lh, rsc->id, nodes,
-                                     constraint->node_attribute,
-                                     (float)constraint->score / INFINITY, flags);
+        if (constraint->score == 0) {
+            continue;
+        }
+        nodes = pcmk__native_merge_weights(constraint->rsc_lh, rsc->id, nodes,
+                                           constraint->node_attribute,
+                                           constraint->score / (float) INFINITY,
+                                           flags);
     }
 
     clear_bit(rsc->flags, pe_rsc_merging);
@@ -512,6 +527,6 @@ group_merge_weights(resource_t * rsc, const char *rhs, GHashTable * nodes, const
 }
 
 void
-group_append_meta(resource_t * rsc, xmlNode * xml)
+group_append_meta(pe_resource_t * rsc, xmlNode * xml)
 {
 }
